"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Promise = require("bluebird");
const http = require("http");
const https = require("https");
const LocatorException = require("./locatorException");
class RequestLocator {
    static getLocatorFactory() {
        return function resourceLocator(parameters) {
            if (typeof parameters === "string") {
                parameters = { url: parameters };
            }
            const url = parameters.url;
            let agent;
            if (url.indexOf('http://') === 0) {
                agent = http;
            }
            else if (url.indexOf('https://') === 0) {
                agent = https;
            }
            else {
                throw new Error(`invalid url: ${url}`);
            }
            const dataExtractor = parameters.dataExtractor ?
                parameters.dataExtractor :
                RequestLocator.DefaultDataExtractor;
            if (!url) {
                throw new Error("must have resource");
            }
            return () => {
                return new Promise((resolve, reject) => {
                    agent.get(url, (res) => {
                        const output = [];
                        res.setEncoding('utf8');
                        res.on('data', (chunk) => {
                            output.push(chunk);
                        });
                        res.on('end', () => {
                            if (200 <= res.statusCode && res.statusCode < 300) {
                                const result = output.join('');
                                resolve(dataExtractor(result));
                            }
                            else {
                                reject(LocatorException.create("BAD_RESPONSE"));
                            }
                        });
                    }).on('error', (err) => {
                        reject(err);
                    });
                });
            };
        };
    }
}
RequestLocator.DefaultDataExtractor = (data) => {
    let locations;
    try {
        locations = JSON.parse(data).servers;
    }
    catch (e) {
        return null;
    }
    if (!locations) {
        return null;
    }
    const location = locations[Math.floor(Math.random() * locations.length)];
    return {
        host: location.address,
        port: location.port
    };
};
exports.RequestLocator = RequestLocator;
//# sourceMappingURL=request.js.map